# classifier_utils.R
# For DDI and HDACi biomarkers

# TGx-HDACi classifier, base_dir, out_dir
classifier_hdaci <- function(biomarker, data_clf, data_heat, data_new, out_path, infile_name, unique_id, name_label, title_plot) {
  data <- data_new[, c(1, 2)]
  names(data)[1] <- "ID"
  file_out_prefix <- paste(biomarker, unique_id, name_label, sep = "_")
  cat(out_path, biomarker, infile_name, unique_id, name_label, file_out_prefix, file = log_file, sep = ", ", fill = TRUE, append = TRUE)
  
  ########################################################################################
  #Heatmap
  ########################################################################################
  z <- merge(data_heat, data, by.x = "ID", by.y = "ID")
  predictions <- data.frame(Name = names(z)[-1])
  z <- merge(data_clf, z, by.x = "ID", by.y = "ID")
  
  ########################################################################################
  #Predictions
  predictions$NonHDACi <- rep(0, nrow(predictions))
  predictions$HDACi <- rep(0, nrow(predictions))
  predictions$Prediction <- rep("Unknown", nrow(predictions))
  
  for(i in 1:nrow(predictions)){
    #i <- 1
    G <- sum(((z[,i+8] - z$HDACi.score)/z$std.dev)^2) - 2*log(0.5) 
    NG <- sum(((z[,i+8] - z$NonHDACi.score)/z$std.dev)^2) - 2*log(0.5) 
    
    probH <- 1/(1 + exp(G/2 - NG/2))
    probNH <- 1 - probH
    
    # predictions$HDACi[i] <- floor(1000*probH)/1000       # This was the original, to "round" to 3 signficant decimal places
    predictions$HDACi[i] <- probH                          # Use this for scientific notation output
    # predictions$HDACi[i] <- round(probH, 12)             # Use this to round to 12 significant decimal places
    predictions$NonHDACi[i] <- probNH
    if(predictions$HDACi[i] > 0.9) predictions$Prediction[i] <- "HDAC"
    if(predictions$NonHDACi[i] > 0.9) predictions$Prediction[i] <- "NonHDAC"
  }
  
  ########################################################################################
  #Heatmap
  x <- as.matrix(z[,-c(1:8)])
  rownames(x) <- as.character(z$ID)
  
  ##############################################################
  #Colours
  my.col <- matrix(rep("white", 2*ncol(x)), ncol = 2)
  my.col[1:10,1] <- rep("red",10)
  my.col[11:20,1] <- rep("blue",10)
  flag <- predictions$HDACi > 0.9
  my.col[flag,2] <- rep("red", length(my.col[flag,2]))
  
  flag <- predictions$NonHDACi > 0.9
  my.col[flag,2] <- rep("blue", length(my.col[flag,2]))
  
  ##############################################################
  #Heatmap
  end.breaks <- c(0, 20, ncol(x))
  y <- NULL
  my.col2 <- NULL
  g <- NULL
  col.blank <- rep(NA, nrow(x))
  pp <- NULL
  
  for(k in 2:length(end.breaks)){
    y <- cbind(y, x[, (end.breaks[k-1]+1):end.breaks[k]], col.blank)
    my.col2 <- rbind(my.col2, my.col[(end.breaks[k-1]+1):end.breaks[k],], c("white", "white"))
    g <- c(g, colnames(x)[(end.breaks[k-1]+1):end.breaks[k]], " ")
  }
  
  y <- y[,-ncol(y)]
  my.col2 <- my.col2[-nrow(my.col2),]
  g <- g[-length(g)]
  colnames(y) <- g
  
  colnames(my.col) <- c("Class", "Prediction")
  colnames(my.col2) <- c("Class", "Prediction")
  
  #############################################################################
  # Heatmap
  dist_gene <- as.dist(1-cor(t(x)))
  my.row <- hclust(dist_gene, method = "average")
  my.row <- as.dendrogram(my.row)
  my.colm <- as.dist(1-cor(x))
  my.colm <- hclust(my.colm, method = "average")
  my.colm <- as.dendrogram(my.colm)
  
  n.col <- 32
  
  plot_file_labels <- get_plot_file_labels()
  # file_extensions <- get_file_extensions()
  file_suffix <- paste(plot_file_labels$heatmap, file_extensions$png, sep = ".")
  png_hdaci_heatmap(out_path, file_out_prefix, file_suffix, y, my.row, n.col, my.col2, title_plot, date_stamp)
    
  # file_suffix <- paste(plot_file_labels$heatmap, file_extensions$pdf, sep = ".")
  # pdf_hdaci_heatmap(out_path, file_out_prefix, file_suffix, y, my.row, n.col, my.col2, title_plot, date_stamp)

  #############################################################################
  # Chemical Hierachical Clustering 
  dist_chem <- dist(t(x))
  dendro <- hclust(dist_chem, method = "average")
  file_suffix <- paste(plot_file_labels$dendro, file_extensions$png, sep = ".")
  png_hdaci_chem_dendro(out_path, file_out_prefix, file_suffix, dendro, title_plot)
  
  #############################################################################
  #Principal Component Analysis
  x_scaled <- (x - z$pc.mean)/z$pc.sd
  file_suffix <- paste(plot_file_labels$pca, file_extensions$png, sep = ".")
  png_hdaci_chem_pca(out_path, file_out_prefix, file_suffix, x_scaled, z, title_plot)

  # file_suffix <- paste(plot_file_labels$pca, file_extensions$pdf, sep = ".")
  # pdf_hdaci_chem_pca(out_path, file_out_prefix, file_suffix, x_scaled, z, title_plot)
  
  # Write output data files
  # txt_file_labels <- get_txt_file_labels()
  fchdataFile <- foldchange_data_to_file(out_path, file_out_prefix, x)
  file_gene_dist <- distance_data_to_file(out_path, file_out_prefix, txt_file_labels$dist_gene, dist_gene)
  file_chem_dist <- distance_data_to_file(out_path, file_out_prefix, txt_file_labels$dist_chem, dist_chem)
  file_prediction <- write_to_tab_del_file(out_path, file_out_prefix, txt_file_labels$prediction, file_extensions$txt, predictions)

  # return (predictions[predictions$Name == name_label, ])
  return (predictions)
}

# General <- function(dat, gs, heatData, pctitle, name_label, outfileName){, base_dir, out_dir
# TGx-DDI classifier
classifier_ddi <- function(biomarker, data_clf, data_heat, data_new, out_path, infile_name, unique_id, name_label, title_plot) {
  data <- data_new[, c(1, 2)]
  names(data)[1] <- "ID"
  file_out_prefix <- paste(biomarker, unique_id, name_label, sep = "_")
  cat(biomarker, out_path, infile_name, unique_id, name_label, file_out_prefix, file = log_file, sep = ", ", fill = TRUE, append = TRUE)
  
  data[,-1] <- log10(2 ^ data[,-1])
  data[,1] <- toupper(data[,1])
  dat <- merge(data_clf, data, by.x = "ID", by.y = "ID")
  
  #Merge Heatmap data
  names(data_heat)[1] <- "ID"
  dat <- merge(dat[,c(1,3,4,7,8)], data_heat, by.x = "ID", by.y = "ID")
  
  #Arranging the data so that the unknown sample is last
  dat <- dat[,c(1:4, 6:ncol(dat), 5)]
  
  #Obtain Predictions
  predictions <- data.frame(Name = names(dat)[5:ncol(dat)])
  predictions$Non_DDI_pvalue <- 0
  predictions$DDI_pvalue <- 0
  predictions$Prediction <- "NonDDI"
  
  cl <- c(rep("red", 13), rep("blue", 16), "white")
  pred <- rep("white", length(cl))
  # pred <- rep("white", length(cl))

  for(k in 5:ncol(dat)){
    G <- sum(((dat[,k] - dat$Genotoxic.score)/dat$my.train.sd)^2) - 2*log(0.5) 
    NG <- sum(((dat[,k] - dat$Non.Genotoxic.score)/dat$my.train.sd)^2) - 2*log(0.5) 
    
    probG <- exp(-G/2)/(exp(-G/2)+exp(-NG/2))
    probNG <- exp(-NG/2)/(exp(-G/2)+exp(-NG/2))
    class <- "Unclassified"
    
    if (is.null(probNG)) {
      print(k)
    }

    if(probG > 0.9){
      class <- "Genotoxic"
      pred[k-4] <- "red"
      predictions$Prediction[k - 4] <- "DDI"
    }
    if(probNG > 0.9){
      class <- "Non-Genotoxic"
      pred[k-4] <- "blue"
    }

    # Use these lines for Scientific Notation    
    predictions$DDI_pvalue[k - 4] <- probG
    predictions$Non_DDI_pvalue[k - 4] <- probNG
    # Use these lines instead for rounding to 12 significant digits
    # predictions$DDI_pvalue[k - 4] <- round(probG, 12)
    # predictions$Non_DDI_pvalue[k - 4] <- round(probNG, 12)
  }
  
  my.col <- cbind(cl, pred)
  x <- as.matrix(dat[,-c(1:4)])
  rownames(x) <- as.character(dat[,1])
  colnames(x) <- c("5 FU", "AraC",  "Bleomycin", "Camptothecin",  "Cisplatin",  "Etoposide",
                   "Hydrogen Peroxide", "Hydroxyurea", "Methotrexate", "MMS",
                   "Potassium.Chromate.VI",  "Sodium Arsenite", "X ray",  "2 DG",  "Apicidin",
                   "Antimycin A", "Cadmium Chloride", "Colchicine", "Docetaxel", "EtOH 2%",
                   "EtOH 4%", "HC toxin", "Heat Shock",  "Oxamflatin", "Paclitaxel",
                   "Thapsigargin",  "TSA", "Tunicamycin", "Vinblastine", name_label) 
  
  #Preparing breaks for the heatmap
  end.breaks <- c(0, 29, ncol(x))
  y <- NULL
  my.col2 <- NULL
  g <- NULL
  col.blank <- rep(NA,nrow(x))
  pp <- NULL
  
  for(k in 2:length(end.breaks)){
    y <- cbind(y, x[, (end.breaks[k-1]+1):end.breaks[k]], col.blank)
    my.col2 <- rbind(my.col2, my.col[(end.breaks[k-1]+1):end.breaks[k],], c("white", "white"))
    g <- c(g, colnames(x)[(end.breaks[k-1]+1):end.breaks[k]], " ")
  }
  
  y <- y[,-ncol(y)]
  my.col2 <- my.col2[-nrow(my.col2),]
  g <- g[-length(g)]
  colnames(y) <- g
  
  colnames(my.col) <- c("Class", "Prediction")
  colnames(my.col2) <- c("Class", "Prediction")
  
  #Row clustering
  dist_gene <- dist(x)
  my.row <- hclust(dist_gene, method = "average")
  my.row <- as.dendrogram(my.row)
  
  n.col <- 17

  # setwd(results_path);
  time_stamp <- format(Sys.time(), "%d-%b-%Y")
  plot_file_labels <- get_plot_file_labels()
  # file_extensions <- get_file_extensions()
  pdf_file = "plots_general.pdf";
  
  if (!is.null(file_out_prefix) && nchar(file_out_prefix) > 0) {
    pdf_file = paste(paste(file_out_prefix, plot_file_labels$plots_all, sep = "_"), 'pdf', sep = ".")
  }
  
  # file_pca <- file.path(base_dir, out_dir, paste("PCA", file_out_prefix, sep = "_"))
  CairoPDF(file = file.path(out_path, pdf_file), width = 11, height = 8.5,
           pointsize = 12, family = "Courier")
  #heatmap3(y, Rowv = my.row, Colv = NA, scale = "row", margins = c(15, 10), 
  #        cexRow=0.7, cexCol=1, col = myRGBColVec(n.col), ColSideColors = my.col2)
  
  plotHeatmap(y, my.row, n.col, my.col2, title_plot, time_stamp)
  
  # Plot cluster and PCA
  row.loc <- apply(x[,1:29],1,mean)
  row.scale <- apply(x[,1:29],1,sd)
  x_scaled <- (x - row.loc)/row.scale
  
  dist_chem <- plotCluster(x_scaled, title_plot, time_stamp)
    
  dev.off()
  # chemicaldistanceFile(file_out_prefix, dist_chem)
  # txt_file_labels <- get_txt_file_labels()
  fchdataFile <- foldchange_data_to_file(out_path, file_out_prefix, x)
  # genedistfile <- genedistance_to_file(out_path, file_out_prefix, dist_gene)
  file_gene_dist <- distance_data_to_file(out_path, file_out_prefix, txt_file_labels$dist_gene, dist_gene)
  file_chem_dist <- distance_data_to_file(out_path, file_out_prefix, txt_file_labels$dist_chem, dist_chem)
  file_prediction <- write_to_tab_del_file(out_path, file_out_prefix, txt_file_labels$prediction, file_extensions$txt, predictions)
  
  ###
  # Repeat heatmap and cluster plots to PNG files
  ###
  heatmapimgfile <- heatmap_plot_png(y, my.row, n.col, my.col2, file_out_prefix, title_plot, time_stamp, out_path)
  clusterimgfile <- cluster_plot_png(x_scaled, file_out_prefix, title_plot, time_stamp, out_path)
  
  df.clssification <- data.frame(class=class, probG = probG, probNG=probNG)
  
  res2X <- 144
  pca_png_file = paste(paste(file_out_prefix, plot_file_labels$pca, sep = "_"), file_extensions$png, sep = ".")
  # png(filename=file.path(out_path, pca_png_file), width=11 * res2X, height=8.5 * res2X, res=res2X, type="cairo")
  png(filename=file.path(out_path, pca_png_file), width=11, height=8.5, unit="in", res=res2X, type="cairo")
  plot_pca(x_scaled, 13, title_plot, time_stamp, colors = c("red", "blud", "dark green"))  # Plot PCA only
  dev.off()
  
  clust_png_file = paste(paste(file_out_prefix, plot_file_labels$dendro, sep = "_"), file_extensions$png, sep = ".")
  # png(filename=file.path(out_path, clust_png_file), width=11 * res2X, height=8.5 * res2X, res=res2X, type="cairo")
  png(filename=file.path(out_path, clust_png_file), width=11, height=8.5, unit="in", res=res2X, type="cairo")
  plot_cluster(x_scaled, title_plot, time_stamp, colors = c("red", "blud", "dark green"))
  dev.off()

  # return (predictions[predictions$Name == name_label, ])
  return (predictions)
}

get_pdf_file_out_suffix <- function(infile_name) {
  file_in_prefix <- sub(".tsv", "", infile_name)
  file_out_suffix <- paste(file_in_prefix, paste(unique_id, "pdf", sep = "."), sep = "_")
  
  return (file_out_suffix)
}
